import re
import logging

from .source import get_source
from .scheme import get_scheme
from .backend import get_backends
from .exceptions import RegistrationError
from .utils import import_by_path


logger = logging.getLogger(__name__)


def register_file(config: dict, path: str, replace: bool=False):
    """ Handle the registration of a single path.
    """
    logger.info(f"Handling '{path}'.")
    source = get_source(config, path)
    scheme = get_scheme(config, path)
    context = scheme.get_context(source, path)

    for pre_handler in get_pre_handlers(config):
        pre_handler(config, path, context)

    try:
        for backend in get_backends(config, path):
            if backend.exists(source, context):
                if replace:
                    logger.info(f"Replacing '{path}'.")
                    backend.register(source, context, replace=True)
                else:
                    raise RegistrationError(f'Object {context} is already registered')
            else:
                logger.info(f"Registering '{path}'.")
                backend.register(source, context, replace=False)
    except Exception as e:
        for error_handler in get_error_handlers(config):
            error_handler(config, path, context, e)
        raise
    else:
        for post_handler in get_post_handlers(config):
            post_handler(config, path, context)

        logger.info(f"Successfully {'replaced' if replace else 'registered'} '{path}'")
        return context


def _get_handlers(config, name):
    handlers = []
    for handler_def in config.get(name, []):
        handler_cls = import_by_path(handler_def['path'])
        handlers.append(
            handler_cls(
                *handler_def.get('args', []),
                **handler_def.get('kwargs', []),
            )
        )

    return handlers


def get_pre_handlers(config):
    return _get_handlers(config, 'pre_handlers')


def get_post_handlers(config):
    return _get_handlers(config, 'post_handlers')


def get_error_handlers(config):
    return _get_handlers(config, 'error_handlers')
